@@ -1,9 +1,9 @@
 import sys
 
-from sqlalchemy.testing import exclusions
 from sqlalchemy.testing.requirements import Requirements
 
 from alembic import util
+from alembic.testing import exclusions
 from alembic.util import sqla_compat
 
 
@@ -43,6 +43,15 @@ class SuiteRequirements(Requirements):
 
         return exclusions.skip_if(doesnt_have_check_uq_constraints)
 
+    @property
+    def sequences(self):
+        """Target database must support SEQUENCEs."""
+
+        return exclusions.only_if(
+            [lambda config: config.db.dialect.supports_sequences],
+            "no sequence support",
+        )
+
     @property
     def foreign_key_match(self):
         return exclusions.open()
@@ -62,21 +71,6 @@ class SuiteRequirements(Requirements):
     def reflects_fk_options(self):
         return exclusions.closed()
 
-    @property
-    def sqlalchemy_issue_3740(self):
-        """Fixes percent sign escaping for paramstyles that don't require it"""
-        return exclusions.skip_if(
-            lambda config: not util.sqla_120,
-            "SQLAlchemy 1.2 or greater required",
-        )
-
-    @property
-    def sqlalchemy_12(self):
-        return exclusions.skip_if(
-            lambda config: not util.sqla_1216,
-            "SQLAlchemy 1.2.16 or greater required",
-        )
-
     @property
     def sqlalchemy_13(self):
         return exclusions.skip_if(
@@ -85,33 +79,10 @@ class SuiteRequirements(Requirements):
         )
 
     @property
-    def sqlalchemy_1115(self):
+    def sqlalchemy_14(self):
         return exclusions.skip_if(
-            lambda config: not util.sqla_1115,
-            "SQLAlchemy 1.1.15 or greater required",
-        )
-
-    @property
-    def sqlalchemy_110(self):
-        return exclusions.skip_if(
-            lambda config: not util.sqla_110,
-            "SQLAlchemy 1.1.0 or greater required",
-        )
-
-    @property
-    def sqlalchemy_issue_4436(self):
-        def check(config):
-            vers = sqla_compat._vers
-
-            if vers == (1, 3, 0, "b1"):
-                return True
-            elif vers >= (1, 2, 16):
-                return False
-            else:
-                return True
-
-        return exclusions.skip_if(
-            check, "SQLAlchemy 1.2.16, 1.3.0b2 or greater required"
+            lambda config: not util.sqla_14,
+            "SQLAlchemy 1.4 or greater required",
         )
 
     @property
@@ -128,15 +99,37 @@ class SuiteRequirements(Requirements):
     @property
     def comments(self):
         return exclusions.only_if(
-            lambda config: sqla_compat._dialect_supports_comments(
-                config.db.dialect
-            )
+            lambda config: config.db.dialect.supports_comments
         )
 
-    @property
-    def comments_api(self):
-        return exclusions.only_if(lambda config: util.sqla_120)
-
     @property
     def alter_column(self):
         return exclusions.open()
+
+    @property
+    def computed_columns(self):
+        return exclusions.closed()
+
+    @property
+    def computed_columns_api(self):
+        return exclusions.only_if(
+            exclusions.BooleanPredicate(sqla_compat.has_computed)
+        )
+
+    @property
+    def identity_columns(self):
+        return exclusions.closed()
+
+    @property
+    def identity_columns_alter(self):
+        return exclusions.closed()
+
+    @property
+    def identity_columns_api(self):
+        return exclusions.only_if(
+            exclusions.BooleanPredicate(sqla_compat.has_identity)
+        )
+
+    @property
+    def supports_identity_on_null(self):
+        return exclusions.closed()
